local tr = aegisub.gettext

script_name = tr"均匀K轴"
script_description = tr"批量给一行字添加均匀进行的K轴"
script_author = "Ted Zyzsdy"
script_version = "0.1.0"

function add_ktags(subs, sel)
	for _, i in ipairs(sel) do
		local line = subs[i]
		local dur = (line.end_time - line.start_time) / 10
		local text_len = utf8len(line.text)
		local one_k_dur = math.floor(dur / text_len)
		local new_text = ""
		for vi=1, text_len, 1 do
			local uchar = utf8sub(line.text, vi, 1)
			new_text = new_text.."{\\k"..one_k_dur.."}"..uchar
		end
		line.text = new_text
		subs[i] = line
	end
	aegisub.set_undo_point(tr"均匀K轴")
end

local function chsize(char)
    if not char then
        fatal("not char")
        return 0
    elseif char > 240 then
        return 4
    elseif char > 225 then
        return 3
    elseif char > 192 then
        return 2
    else
        return 1
    end
end

function utf8len(str)
    local len = 0
    local currentIndex = 1
    while currentIndex <= #str do
        local char = string.byte(str, currentIndex)
        currentIndex = currentIndex + chsize(char)
        len = len +1
    end
    return len
end

function utf8sub(str, startChar, numChars)
    local startIndex = 1
    while startChar > 1 do
        local char = string.byte(str, startIndex)
        startIndex = startIndex + chsize(char)
        startChar = startChar - 1
    end

    local currentIndex = startIndex

    while numChars > 0 and currentIndex <= #str do
        local char = string.byte(str, currentIndex)
        currentIndex = currentIndex + chsize(char)
        numChars = numChars - 1
    end
    return str:sub(startIndex, currentIndex - 1)
end

aegisub.register_macro(script_name, script_description, add_ktags)